@@ -19,6 +19,7 @@
  */
 package com.eteks.sweethome3d.applet;
 
+import java.io.File;
 import java.io.IOException;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
@@ -178,6 +179,8 @@ public class SweetHome3DApplet extends JApplet {
   public void init() {
     if (!isJava5OrSuperior()) {
       showText(getLocalizedString("requirementsMessage"));
+    } else if (!getDocumentBase().getHost().equals(getCodeBase().getHost())) {
+      showText(getLocalizedString("unauthorizedHostError"));
     } else {
       createAppletApplication();
     }
@@ -267,27 +270,48 @@ public class SweetHome3DApplet extends JApplet {
     try {
       applicationClassName = getApplicationClassName();
       Class sweetHome3DAppletClass = SweetHome3DApplet.class;
-      List java3DFiles = new ArrayList(Arrays.asList(new String [] {
-          "j3dcore.jar", // Main Java 3D jars
-          "vecmath.jar",
-          "j3dutils.jar",
-          "macosx/gluegen-rt.jar", // Mac OS X jars and DLLs
-          "macosx/jogl.jar",
-          "macosx/libgluegen-rt.jnilib",
-          "macosx/libjogl.jnilib",
-          "macosx/libjogl_awt.jnilib",
-          "macosx/libjogl_cg.jnilib"}));
-      if ("64".equals(System.getProperty("sun.arch.data.model"))) {
-        java3DFiles.add("linux/x64/libj3dcore-ogl.so"); // Linux DLL
-        java3DFiles.add("windows/x64/j3dcore-ogl.dll"); // Windows DLL
+      List java3DFiles = new ArrayList();
+      if (!System.getProperty("os.name").startsWith("Mac OS X")
+          || System.getProperty("java.version").startsWith("1.5")
+          || System.getProperty("java.version").startsWith("1.6")) {
+        java3DFiles.addAll(Arrays.asList(new String [] {
+            "j3dcore.jar", // Main Java 3D jars
+            "vecmath.jar",
+            "j3dutils.jar",
+            "macosx/gluegen-rt.jar", // Mac OS X jars and DLLs
+            "macosx/jogl.jar",
+            "macosx/libgluegen-rt.jnilib",
+            "macosx/libjogl.jnilib",
+            "macosx/libjogl_awt.jnilib",
+            "macosx/libjogl_cg.jnilib"}));
       } else {
-        java3DFiles.add("linux/i386/libj3dcore-ogl.so"); // Linux DLLs
-        java3DFiles.add("linux/i386/libj3dcore-ogl-cg.so");
-        java3DFiles.add("windows/i386/j3dcore-d3d.dll"); // Windows DLLs
-        java3DFiles.add("windows/i386/j3dcore-ogl.dll");
-        java3DFiles.add("windows/i386/j3dcore-ogl-cg.dll");
-        java3DFiles.add("windows/i386/j3dcore-ogl-chk.dll");
+        java3DFiles.addAll(Arrays.asList(new String [] {
+            "macosx/java3d-1.6/j3dcore.jar", // Mac OS X Java 3D 1.6 jars and DLLs
+            "macosx/java3d-1.6/vecmath.jar",
+            "macosx/java3d-1.6/j3dutils.jar",
+            "macosx/java3d-1.6/gluegen.jar", 
+            "macosx/java3d-1.6/jogl-java3d.jar",
+            "macosx/java3d-1.6/libgluegen-rt.jnilib",
+            "macosx/java3d-1.6/libjogl_desktop.jnilib",
+            "macosx/java3d-1.6/libnativewindow_awt.jnilib",
+            "macosx/java3d-1.6/libnativewindow_macosx.jnilib"}));
+        // Disable JOGL library loader
+        System.setProperty("jogamp.gluegen.UseTempJarCache", "false");
+        System.setProperty("com.eteks.sweethome3d.j3d.useOffScreen3DView", "true");
       }
+      if ("64".equals(System.getProperty("sun.arch.data.model"))) {
+        java3DFiles.add("linux/x64/libj3dcore-ogl.so"); // Linux 64 bits DLLs
+        java3DFiles.add("windows/x64/j3dcore-ogl.dll"); // Windows 64 bits DLLs
+      } else {
+        java3DFiles.addAll(Arrays.asList(new String [] {
+            "linux/i386/libj3dcore-ogl.so", // Linux 32 bits DLLs
+            "linux/i386/libj3dcore-ogl-cg.so", // Windows 32 bits DLLs
+            "windows/i386/j3dcore-d3d.dll",
+            "windows/i386/j3dcore-ogl.dll",
+            "windows/i386/j3dcore-ogl-cg.dll",
+            "windows/i386/j3dcore-ogl-chk.dll"}));
+      }
+      
       List applicationPackages = new ArrayList(Arrays.asList(new String [] {
           "com.eteks.sweethome3d",
           "javax.media",
@@ -295,7 +319,10 @@ public class SweetHome3DApplet extends JApplet {
           "com.sun.j3d",
           "com.sun.opengl",
           "com.sun.gluegen.runtime",
+          "com.jogamp",
+          "jogamp",
           "javax.media.opengl",
+          "javax.media.nativewindow",
           "com.sun.media",
           "com.ibm.media",
           "jmpapps.util",
@@ -317,10 +344,19 @@ public class SweetHome3DApplet extends JApplet {
         applicationPackages.add(applicationClassPackageBase);
       }
       
-      ClassLoader extensionsClassLoader = new ExtensionsClassLoader(
-          sweetHome3DAppletClass.getClassLoader(), sweetHome3DAppletClass.getProtectionDomain(),
-          (String [])java3DFiles.toArray(new String [java3DFiles.size()]), 
-          (String [])applicationPackages.toArray(new String [applicationPackages.size()]));
+      ClassLoader extensionsClassLoader = System.getProperty("os.name").startsWith("Windows")
+          ? new ExtensionsClassLoader(
+              sweetHome3DAppletClass.getClassLoader(), sweetHome3DAppletClass.getProtectionDomain(),
+              (String [])java3DFiles.toArray(new String [java3DFiles.size()]), null, 
+              (String [])applicationPackages.toArray(new String [applicationPackages.size()]),
+              // Use cache under Windows because temporary files tagged as deleteOnExit can't 
+              // be deleted if they are still opened when program exits (as DLLs can't be shared 
+              // by two class loaders, manage only Jar files in cache)
+              new File(System.getProperty("java.io.tmpdir")), applicationClassName + "-cache-", true)  
+          : new ExtensionsClassLoader(
+              sweetHome3DAppletClass.getClassLoader(), sweetHome3DAppletClass.getProtectionDomain(),
+              (String [])java3DFiles.toArray(new String [java3DFiles.size()]), 
+              (String [])applicationPackages.toArray(new String [applicationPackages.size()]));
       startApplication(applicationClassName, extensionsClassLoader);
     } catch (AccessControlException ex) {
       String runWithoutSignature = getParameter("runWithoutSignature");
