@@ -19,12 +19,12 @@
  */
 package com.eteks.sweethome3d.viewcontroller;
 
-import java.awt.geom.Rectangle2D;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
+import java.lang.ref.WeakReference;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collection;
+import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
 
@@ -41,6 +41,8 @@ import com.eteks.sweethome3d.model.Level;
 import com.eteks.sweethome3d.model.ObserverCamera;
 import com.eteks.sweethome3d.model.Room;
 import com.eteks.sweethome3d.model.Selectable;
+import com.eteks.sweethome3d.model.SelectionEvent;
+import com.eteks.sweethome3d.model.SelectionListener;
 import com.eteks.sweethome3d.model.UserPreferences;
 import com.eteks.sweethome3d.model.Wall;
 
@@ -76,7 +78,7 @@ public class HomeController3D implements Controller {
     this.contentManager = contentManager;
     this.undoSupport = undoSupport;
     // Initialize states
-    this.topCameraState = new TopCameraState();
+    this.topCameraState = new TopCameraState(preferences);
     this.observerCameraState = new ObserverCameraState();
     // Set default state 
     setCameraState(home.getCamera() == home.getTopCamera() 
@@ -227,7 +229,22 @@ public class HomeController3D implements Controller {
     storedCameras.add(0, camera);
     this.home.setStoredCameras(storedCameras);
   }
-  
+
+  /**
+   * Deletes the given list of cameras from the ones stored in home.
+   */
+  public void deleteCameras(List<Camera> cameras) {
+    List<Camera> homeStoredCameras = this.home.getStoredCameras();
+    // Build a list of cameras that will contain only the cameras not in the camera list in parameter
+    ArrayList<Camera> storedCameras = new ArrayList<Camera>(homeStoredCameras.size() - cameras.size());
+    for (Camera camera : homeStoredCameras) {
+      if (!cameras.contains(camera)) {
+        storedCameras.add(camera);
+      }
+    }
+    this.home.setStoredCameras(storedCameras);
+  }
+
   /**
    * Makes all levels visible.
    */
@@ -335,31 +352,29 @@ public class HomeController3D implements Controller {
    * Top camera controller state. 
    */
   private class TopCameraState extends CameraControllerState {
-    private final Rectangle2D MIN_BOUNDS = new Rectangle2D.Float(0, 0, 1000, 1000);
+    private final float MIN_WIDTH  = 1000;
+    private final float MIN_DEPTH  = 1000;
+    private final float MIN_HEIGHT = 20;
     
     private Camera      topCamera;
-    private Rectangle2D homeBounds;
-    private float       minDistanceToHomeCenter;
-    private PropertyChangeListener levelVisibilityChangeListener = new PropertyChangeListener() {
+    private float []    aerialViewBoundsLowerPoint;
+    private float []    aerialViewBoundsUpperPoint;
+    private float       minDistanceToAerialViewCenter;
+    private float       maxDistanceToAerialViewCenter;
+    private boolean     aerialViewCenteredOnSelectionEnabled;
+    private PropertyChangeListener objectChangeListener = new PropertyChangeListener() {
         public void propertyChange(PropertyChangeEvent ev) {
-          if (Level.Property.VISIBLE.name().equals(ev.getPropertyName())) {
-            updateCameraFromHomeBounds();
-          }
+          updateCameraFromHomeBounds(false);
         }
       };
     private CollectionListener<Level> levelsListener = new CollectionListener<Level>() {
         public void collectionChanged(CollectionEvent<Level> ev) {
           if (ev.getType() == CollectionEvent.Type.ADD) {
-            ev.getItem().addPropertyChangeListener(levelVisibilityChangeListener);
+            ev.getItem().addPropertyChangeListener(objectChangeListener);
           } else if (ev.getType() == CollectionEvent.Type.DELETE) {
-            ev.getItem().removePropertyChangeListener(levelVisibilityChangeListener);
+            ev.getItem().removePropertyChangeListener(objectChangeListener);
           } 
-          updateCameraFromHomeBounds();
-        }
-      };
-    private PropertyChangeListener objectChangeListener = new PropertyChangeListener() {
-        public void propertyChange(PropertyChangeEvent ev) {
-          updateCameraFromHomeBounds();
+          updateCameraFromHomeBounds(false);
         }
       };
     private CollectionListener<Wall> wallsListener = new CollectionListener<Wall>() {
@@ -369,17 +384,20 @@ public class HomeController3D implements Controller {
           } else if (ev.getType() == CollectionEvent.Type.DELETE) {
             ev.getItem().removePropertyChangeListener(objectChangeListener);
           } 
-          updateCameraFromHomeBounds();
+          updateCameraFromHomeBounds(false);
         }
       };
     private CollectionListener<HomePieceOfFurniture> furnitureListener = new CollectionListener<HomePieceOfFurniture>() {
         public void collectionChanged(CollectionEvent<HomePieceOfFurniture> ev) {
           if (ev.getType() == CollectionEvent.Type.ADD) {
             ev.getItem().addPropertyChangeListener(objectChangeListener);
+            updateCameraFromHomeBounds(home.getFurniture().size() == 1
+                && home.getWalls().isEmpty()
+                && home.getRooms().isEmpty());
           } else if (ev.getType() == CollectionEvent.Type.DELETE) {
             ev.getItem().removePropertyChangeListener(objectChangeListener);
+            updateCameraFromHomeBounds(false);
           } 
-          updateCameraFromHomeBounds();
         }
       };
     private CollectionListener<Room> roomsListener = new CollectionListener<Room>() {
@@ -389,16 +407,27 @@ public class HomeController3D implements Controller {
           } else if (ev.getType() == CollectionEvent.Type.DELETE) {
             ev.getItem().removePropertyChangeListener(objectChangeListener);
           } 
-          updateCameraFromHomeBounds();
+          updateCameraFromHomeBounds(false);
         }
       };
+    private SelectionListener selectionListener = new SelectionListener() {
+        public void selectionChanged(SelectionEvent ev) {
+          updateCameraFromHomeBounds(false);
+        }
+      };
+
+    public TopCameraState(UserPreferences preferences) {
+      this.aerialViewCenteredOnSelectionEnabled = preferences.isAerialViewCenteredOnSelectionEnabled();
+      preferences.addPropertyChangeListener(UserPreferences.Property.AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED, 
+          new UserPreferencesChangeListener(this));
+    }
 
     @Override
     public void enter() {
       this.topCamera = home.getCamera();
-      updateCameraFromHomeBounds();
+      updateCameraFromHomeBounds(false);
       for (Level level : home.getLevels()) {
-        level.addPropertyChangeListener(this.levelVisibilityChangeListener);
+        level.addPropertyChangeListener(this.objectChangeListener);
       }
       home.addLevelsListener(this.levelsListener);
       for (Wall wall : home.getWalls()) {
@@ -413,185 +442,245 @@ public class HomeController3D implements Controller {
         room.addPropertyChangeListener(this.objectChangeListener);
       }
       home.addRoomsListener(this.roomsListener);
+      home.addSelectionListener(this.selectionListener);
+    }
+    
+    /**
+     * Sets whether aerial view should be centered on selection or not.
+     */
+    public void setAerialViewCenteredOnSelectionEnabled(boolean aerialViewCenteredOnSelectionEnabled) {
+      this.aerialViewCenteredOnSelectionEnabled = aerialViewCenteredOnSelectionEnabled;
+      updateCameraFromHomeBounds(false);
     }
     
     /**
      * Updates camera location from home bounds.
      */
-    private void updateCameraFromHomeBounds() {
-      if (this.homeBounds == null) {
-        this.homeBounds = getHomeBounds();
+    private void updateCameraFromHomeBounds(boolean firstPieceOfFurnitureAddedToEmptyHome) {
+      if (this.aerialViewBoundsLowerPoint == null) {
+        updateAerialViewBounds(this.aerialViewCenteredOnSelectionEnabled);
       }
-      float distanceToCenter = (float)Math.sqrt(Math.pow(this.homeBounds.getCenterX() - this.topCamera.getX(), 2) 
-          + Math.pow(this.homeBounds.getCenterY() - this.topCamera.getY(), 2) 
-          + Math.pow(this.topCamera.getZ(), 2));
-      this.homeBounds = getHomeBounds();
-      this.minDistanceToHomeCenter = getMinDistanceToHomeCenter(this.homeBounds);
-      placeCameraAt(distanceToCenter);
+      float distanceToCenter = getCameraToAerialViewCenterDistance();
+      updateAerialViewBounds(this.aerialViewCenteredOnSelectionEnabled);
+      updateCameraIntervalToAerialViewCenter();
+      placeCameraAt(distanceToCenter, firstPieceOfFurnitureAddedToEmptyHome);
     }
 
     /**
-     * Returns home bounds that includes walls, furniture and rooms.
+     * Returns the distance between the current camera location and home bounds center.
      */
-    private Rectangle2D getHomeBounds() {
+    private float getCameraToAerialViewCenterDistance() {
+      return (float)Math.sqrt(Math.pow((this.aerialViewBoundsLowerPoint [0] + this.aerialViewBoundsUpperPoint [0]) / 2 - this.topCamera.getX(), 2) 
+          + Math.pow((this.aerialViewBoundsLowerPoint [1] + this.aerialViewBoundsUpperPoint [1]) / 2 - this.topCamera.getY(), 2) 
+          + Math.pow((this.aerialViewBoundsLowerPoint [2] + this.aerialViewBoundsUpperPoint [2]) / 2 - this.topCamera.getZ(), 2));
+    }
+
+    /**
+     * Sets the bounds that includes walls, furniture and rooms, or only selected items 
+     * if <code>centerOnSelection</code> is <code>true</code>.
+     */
+    private void updateAerialViewBounds(boolean centerOnSelection) {
+      this.aerialViewBoundsLowerPoint = 
+      this.aerialViewBoundsUpperPoint = null;
+      List<Selectable> selectedItems = Collections.emptyList();
+      if (centerOnSelection) { 
+        selectedItems = new ArrayList<Selectable>();
+        for (Selectable item : home.getSelectedItems()) {
+          if (item instanceof Elevatable 
+              && isItemAtVisibleLevel((Elevatable)item)
+              && (!(item instanceof HomePieceOfFurniture)
+                  || ((HomePieceOfFurniture)item).isVisible())) {
+            selectedItems.add(item);
+          }
+        }
+      }
+      boolean selectionEmpty = selectedItems.size() == 0 || !centerOnSelection;
+
       // Compute plan bounds to include rooms, walls and furniture
-      Rectangle2D homeBounds = null;
       boolean containsVisibleWalls = false;
-      for (Wall wall : home.getWalls()) {
-        if (isItemAtAVisibleLevel(wall)) {
+      for (Wall wall : selectionEmpty
+                           ? home.getWalls()
+                           : Home.getWallsSubList(selectedItems)) {
+        if (isItemAtVisibleLevel(wall)) {
           containsVisibleWalls = true;
+          
+          float wallElevation = wall.getLevel() != null 
+              ? wall.getLevel().getElevation() 
+              : 0;
+          float minZ = selectionEmpty
+              ? 0
+              : wallElevation;
+          
+          Float height = wall.getHeight();
+          float maxZ;
+          if (height != null) {
+            maxZ = wallElevation + height;
+          } else {
+            maxZ = wallElevation + home.getWallHeight();
+          }
+          Float heightAtEnd = wall.getHeightAtEnd();
+          if (heightAtEnd != null) {
+            maxZ = Math.max(maxZ, wallElevation + heightAtEnd);
+          }
           for (float [] point : wall.getPoints()) {
-            homeBounds = updateHomeBounds(homeBounds, point [0], point [1]);
-          }
-        }
-      }
-      for (HomePieceOfFurniture piece : home.getFurniture()) {
-        if (piece.isVisible() && isItemAtAVisibleLevel(piece)) {
-          for (float [] point : piece.getPoints()) {
-            homeBounds = updateHomeBounds(homeBounds, point [0], point [1]);
-          }
-        }
-      }
-      for (Room room : home.getRooms()) {
-        if (isItemAtAVisibleLevel(room)) {
-          for (float [] point : room.getPoints()) {
-            homeBounds = updateHomeBounds(homeBounds, point [0], point [1]);
+            updateAerialViewBounds(point [0], point [1], minZ, maxZ);
           }
         }
       }
 
-      if (homeBounds != null) {
-        if (!containsVisibleWalls) {
-          // If home contains only rooms and furniture don't fix minimum size
-          return homeBounds;
-        } else {
-          // Ensure plan bounds are always minimum 10 meters wide centered in middle of 3D view
-          return new Rectangle2D.Float(
-              (float)(MIN_BOUNDS.getWidth() < homeBounds.getWidth() 
-                          ? homeBounds.getMinX()
-                          : homeBounds.getCenterX() - MIN_BOUNDS.getWidth() / 2), 
-              (float)(MIN_BOUNDS.getHeight() < homeBounds.getHeight() 
-                          ? homeBounds.getMinY()
-                          : homeBounds.getCenterY() - MIN_BOUNDS.getHeight() / 2), 
-              (float)Math.max(MIN_BOUNDS.getWidth(), homeBounds.getWidth()), 
-              (float)Math.max(MIN_BOUNDS.getHeight(), homeBounds.getHeight()));
+      for (HomePieceOfFurniture piece : selectionEmpty 
+                                            ? home.getFurniture()
+                                            : Home.getFurnitureSubList(selectedItems)) {
+        if (piece.isVisible() && isItemAtVisibleLevel(piece)) {
+          float minZ;
+          float maxZ;
+          if (selectionEmpty) {
+            minZ = Math.max(0, piece.getGroundElevation());
+            maxZ = Math.max(0, piece.getGroundElevation() + piece.getHeight());
+          } else {
+            minZ = piece.getGroundElevation();
+            maxZ = piece.getGroundElevation() + piece.getHeight();
+          }
+          for (float [] point : piece.getPoints()) {
+            updateAerialViewBounds(point [0], point [1], minZ, maxZ);
+          }
         }
+      }
+      
+      for (Room room : selectionEmpty 
+                           ? home.getRooms()
+                           : Home.getRoomsSubList(selectedItems)) {
+        if (isItemAtVisibleLevel(room)) {
+          float minZ = 0;
+          float maxZ = MIN_HEIGHT;
+          Level roomLevel = room.getLevel();
+          if (roomLevel != null) {
+            minZ = roomLevel.getElevation() - roomLevel.getFloorThickness();
+            maxZ = roomLevel.getElevation();
+            if (selectionEmpty) {
+              minZ = Math.max(0, minZ);
+              maxZ = Math.max(MIN_HEIGHT, roomLevel.getElevation());
+            }
+          }
+          for (float [] point : room.getPoints()) {
+            updateAerialViewBounds(point [0], point [1], minZ, maxZ);
+          }
+        }
+      }
+      
+      if (this.aerialViewBoundsLowerPoint == null) {
+        this.aerialViewBoundsLowerPoint = new float [] {0, 0, 0};
+        this.aerialViewBoundsUpperPoint = new float [] {MIN_WIDTH, MIN_DEPTH, MIN_HEIGHT};
+      } else if (containsVisibleWalls && selectionEmpty) {
+        // If home contains walls, ensure bounds are always minimum 10 meters wide centered in middle of 3D view
+        if (MIN_WIDTH > this.aerialViewBoundsUpperPoint [0] - this.aerialViewBoundsLowerPoint [0]) {
+          this.aerialViewBoundsLowerPoint [0] = (this.aerialViewBoundsLowerPoint [0] + this.aerialViewBoundsUpperPoint [0]) / 2 - MIN_WIDTH / 2;
+          this.aerialViewBoundsUpperPoint [0] = this.aerialViewBoundsLowerPoint [0] + MIN_WIDTH;
+        }
+        if (MIN_DEPTH > this.aerialViewBoundsUpperPoint [1] - this.aerialViewBoundsLowerPoint [1]) {
+          this.aerialViewBoundsLowerPoint [1] = (this.aerialViewBoundsLowerPoint [1] + this.aerialViewBoundsUpperPoint [1]) / 2 - MIN_DEPTH / 2;
+          this.aerialViewBoundsUpperPoint [1] = this.aerialViewBoundsLowerPoint [1] + MIN_DEPTH;
+        }
+        if (MIN_HEIGHT > this.aerialViewBoundsUpperPoint [2] - this.aerialViewBoundsLowerPoint [2]) {
+          this.aerialViewBoundsLowerPoint [2] = (this.aerialViewBoundsLowerPoint [2] + this.aerialViewBoundsUpperPoint [2]) / 2 - MIN_HEIGHT / 2;
+          this.aerialViewBoundsUpperPoint [2] = this.aerialViewBoundsLowerPoint [2] + MIN_HEIGHT;
+        }
+      }
+    }
+
+    /**
+     * Adds the point at the given coordinates to aerial view bounds.
+     */
+    private void updateAerialViewBounds(float x, float y, float minZ, float maxZ) {
+      if (this.aerialViewBoundsLowerPoint == null) {
+        this.aerialViewBoundsLowerPoint = new float [] {x, y, minZ};
+        this.aerialViewBoundsUpperPoint = new float [] {x, y, maxZ};
       } else {
-        return MIN_BOUNDS;
+        this.aerialViewBoundsLowerPoint [0] = Math.min(this.aerialViewBoundsLowerPoint [0], x); 
+        this.aerialViewBoundsUpperPoint [0] = Math.max(this.aerialViewBoundsUpperPoint [0], x);
+        this.aerialViewBoundsLowerPoint [1] = Math.min(this.aerialViewBoundsLowerPoint [1], y); 
+        this.aerialViewBoundsUpperPoint [1] = Math.max(this.aerialViewBoundsUpperPoint [1], y);
+        this.aerialViewBoundsLowerPoint [2] = Math.min(this.aerialViewBoundsLowerPoint [2], minZ); 
+        this.aerialViewBoundsUpperPoint [2] = Math.max(this.aerialViewBoundsUpperPoint [2], maxZ);
       }
     }
 
     /**
      * Returns <code>true</code> if the given <code>item</code> is at a visible level.
      */
-    private boolean isItemAtAVisibleLevel(Elevatable item) {
+    private boolean isItemAtVisibleLevel(Elevatable item) {
       return item.getLevel() == null || item.getLevel().isVisible();
     }
     
     /**
-     * Adds the point at the given coordinates to <code>homeBounds</code>.
+     * Updates the minimum and maximum distances of the camera to the center of the aerial view.
      */
-    private Rectangle2D updateHomeBounds(Rectangle2D homeBounds,
-                                         float x, float y) {
-      if (homeBounds == null) {
-        homeBounds = new Rectangle2D.Float(x, y, 0, 0);
-      } else {
-        homeBounds.add(x, y);
-      }
-      return homeBounds;
+    private void updateCameraIntervalToAerialViewCenter() {  
+      float homeBoundsWidth = this.aerialViewBoundsUpperPoint [0] - this.aerialViewBoundsLowerPoint [0];
+      float homeBoundsDepth = this.aerialViewBoundsUpperPoint [1] - this.aerialViewBoundsLowerPoint [1];
+      float homeBoundsHeight = this.aerialViewBoundsUpperPoint [2] - this.aerialViewBoundsLowerPoint [2];
+      float halfDiagonal = (float)Math.sqrt(homeBoundsWidth * homeBoundsWidth 
+          + homeBoundsDepth * homeBoundsDepth 
+          + homeBoundsHeight * homeBoundsHeight) / 2;
+      this.minDistanceToAerialViewCenter = halfDiagonal * 1.05f;
+      this.maxDistanceToAerialViewCenter = Math.max(5 * this.minDistanceToAerialViewCenter, 1000);
     }
-
-    /**
-     * Returns the minimum distance of the camera to home center.
-     */
-    private float getMinDistanceToHomeCenter(Rectangle2D homeBounds) {
-      float maxHeight = 0;
-      Collection<Wall> walls = home.getWalls();
-      if (walls.isEmpty()) {
-        // If home contains no wall, search the max height of the highest piece
-        for (HomePieceOfFurniture piece : home.getFurniture()) {
-          if (piece.isVisible()) {
-            maxHeight = Math.max(maxHeight, piece.getGroundElevation() + piece.getHeight());
-          }
-        }
-      } else {
-         // Search the max height of the highest wall
-        for (Wall wall : walls) {
-          Float height = wall.getHeight();
-          Level wallLevel = wall.getLevel();
-          float levelElevation = wallLevel == null 
-              ? 0
-              : wallLevel.getElevation(); 
-          if (height != null) {
-            maxHeight = Math.max(maxHeight, levelElevation + height);
-          }
-          Float heightAtEnd = wall.getHeightAtEnd();
-          if (heightAtEnd != null) {
-            maxHeight = Math.max(maxHeight, levelElevation + heightAtEnd);
-          }
-        }
-      }
-      if (maxHeight > 0) {
-        maxHeight = Math.max(10, maxHeight);
-      } else {
-        maxHeight = home.getWallHeight();        
-      }
-      double halfDiagonal = Math.sqrt(homeBounds.getWidth() * homeBounds.getWidth() + homeBounds.getHeight() * homeBounds.getHeight()) / 2;
-      return (float)Math.sqrt(maxHeight * maxHeight + halfDiagonal * halfDiagonal) * 1.1f;
-    }
-    
+       
     @Override
     public void moveCamera(float delta) {
       // Use a 5 times bigger delta for top camera move
       delta *= 5;
-      float newDistanceToCenter = (float)Math.sqrt(Math.pow(this.homeBounds.getCenterX() - this.topCamera.getX(), 2) 
-          + Math.pow(this.homeBounds.getCenterY() - this.topCamera.getY(), 2) 
-          + Math.pow(this.topCamera.getZ(), 2)) - delta;
-      placeCameraAt(newDistanceToCenter);
+      float newDistanceToCenter = getCameraToAerialViewCenterDistance() - delta;
+      placeCameraAt(newDistanceToCenter, false);
     }
 
-    public void placeCameraAt(float distanceToCenter) {
+    public void placeCameraAt(float distanceToCenter, boolean firstPieceOfFurnitureAddedToEmptyHome) {
       // Check camera is always outside the sphere centered in home center and with a radius equal to minimum distance   
-      distanceToCenter = Math.max(distanceToCenter, this.minDistanceToHomeCenter);
+      distanceToCenter = Math.max(distanceToCenter, this.minDistanceToAerialViewCenter);
       // Check camera isn't too far
-      distanceToCenter = Math.min(distanceToCenter, 5 * this.minDistanceToHomeCenter);
+      distanceToCenter = Math.min(distanceToCenter, this.maxDistanceToAerialViewCenter);
+      if (firstPieceOfFurnitureAddedToEmptyHome) {
+        // Get closer to the first piece of furniture added to an empty home when that is small
+        distanceToCenter = Math.min(distanceToCenter, 3 * this.minDistanceToAerialViewCenter);
+      }
       double distanceToCenterAtGroundLevel = distanceToCenter * Math.cos(this.topCamera.getPitch());
-      this.topCamera.setX((float)this.homeBounds.getCenterX() + (float)(Math.sin(this.topCamera.getYaw()) 
-          * distanceToCenterAtGroundLevel));
-      this.topCamera.setY((float)this.homeBounds.getCenterY() - (float)(Math.cos(this.topCamera.getYaw()) 
-          * distanceToCenterAtGroundLevel));
-      this.topCamera.setZ((float)Math.sin(this.topCamera.getPitch()) * distanceToCenter);
+      this.topCamera.setX((this.aerialViewBoundsLowerPoint [0] + this.aerialViewBoundsUpperPoint [0]) / 2 
+          + (float)(Math.sin(this.topCamera.getYaw()) * distanceToCenterAtGroundLevel));
+      this.topCamera.setY((this.aerialViewBoundsLowerPoint [1] + this.aerialViewBoundsUpperPoint [1]) / 2 
+          - (float)(Math.cos(this.topCamera.getYaw()) * distanceToCenterAtGroundLevel));
+      this.topCamera.setZ((this.aerialViewBoundsLowerPoint [2] + this.aerialViewBoundsUpperPoint [2]) / 2 
+          + (float)Math.sin(this.topCamera.getPitch()) * distanceToCenter);
     }
 
     @Override
     public void rotateCameraYaw(float delta) {
-      float  newYaw = this.topCamera.getYaw() + delta;
-      double distanceToCenterAtGroundLevel = this.topCamera.getZ() / Math.tan(this.topCamera.getPitch());
+      float newYaw = this.topCamera.getYaw() + delta;
+      double distanceToCenterAtGroundLevel = getCameraToAerialViewCenterDistance() * Math.cos(this.topCamera.getPitch());
       // Change camera yaw and location so user turns around home
       this.topCamera.setYaw(newYaw); 
-      this.topCamera.setX((float)this.homeBounds.getCenterX() + (float)(Math.sin(newYaw) * distanceToCenterAtGroundLevel));
-      this.topCamera.setY((float)this.homeBounds.getCenterY() - (float)(Math.cos(newYaw) * distanceToCenterAtGroundLevel));
+      this.topCamera.setX((this.aerialViewBoundsLowerPoint [0] + this.aerialViewBoundsUpperPoint [0]) / 2 
+          + (float)(Math.sin(newYaw) * distanceToCenterAtGroundLevel));
+      this.topCamera.setY((this.aerialViewBoundsLowerPoint [1] + this.aerialViewBoundsUpperPoint [1]) / 2 
+          - (float)(Math.cos(newYaw) * distanceToCenterAtGroundLevel));
     }
     
     @Override
     public void rotateCameraPitch(float delta) {
       float newPitch = this.topCamera.getPitch() + delta;
-      // Check new pitch is between PI / 2 and PI / 16  
-      newPitch = Math.max(newPitch, (float)Math.PI / 16);
+      // Check new pitch is between 0 and PI / 2  
+      newPitch = Math.max(newPitch, (float)0);
       newPitch = Math.min(newPitch, (float)Math.PI / 2);
       // Compute new z to keep the same distance to view center
-      double cameraToBoundsCenterDistance = Math.sqrt(Math.pow(this.topCamera.getX() - this.homeBounds.getCenterX(), 2)
-          + Math.pow(this.topCamera.getY() - this.homeBounds.getCenterY(), 2)
-          + Math.pow(this.topCamera.getZ(), 2));
-      float newZ = (float)(cameraToBoundsCenterDistance * Math.sin(newPitch));
-      double distanceToCenterAtGroundLevel = newZ / Math.tan(newPitch);
+      double distanceToCenter = getCameraToAerialViewCenterDistance();
+      double distanceToCenterAtGroundLevel = distanceToCenter * Math.cos(newPitch);
       // Change camera pitch 
       this.topCamera.setPitch(newPitch); 
-      this.topCamera.setX((float)this.homeBounds.getCenterX() + (float)(Math.sin(this.topCamera.getYaw()) 
-          * distanceToCenterAtGroundLevel));
-      this.topCamera.setY((float)this.homeBounds.getCenterY() - (float)(Math.cos(this.topCamera.getYaw()) 
-          * distanceToCenterAtGroundLevel));
-      this.topCamera.setZ(newZ);
+      this.topCamera.setX((this.aerialViewBoundsLowerPoint [0] + this.aerialViewBoundsUpperPoint [0]) / 2 
+          + (float)(Math.sin(this.topCamera.getYaw()) * distanceToCenterAtGroundLevel));
+      this.topCamera.setY((this.aerialViewBoundsLowerPoint [1] + this.aerialViewBoundsUpperPoint [1]) / 2 
+          - (float)(Math.cos(this.topCamera.getYaw()) * distanceToCenterAtGroundLevel));
+      this.topCamera.setZ((this.aerialViewBoundsLowerPoint [2] + this.aerialViewBoundsUpperPoint [2]) / 2 
+          + (float)(distanceToCenter * Math.sin(newPitch)));
     }
     
     @Override
@@ -599,7 +688,7 @@ public class HomeController3D implements Controller {
       this.topCamera.setCamera(camera);
       this.topCamera.setTime(camera.getTime());
       this.topCamera.setLens(camera.getLens());
-      updateCameraFromHomeBounds();
+      updateCameraFromHomeBounds(false);
     }
     
     @Override
@@ -617,13 +706,37 @@ public class HomeController3D implements Controller {
         room.removePropertyChangeListener(this.objectChangeListener);
       }
       home.removeRoomsListener(this.roomsListener);
-      for (Level room : home.getLevels()) {
-        room.removePropertyChangeListener(this.levelVisibilityChangeListener);
+      for (Level level : home.getLevels()) {
+        level.removePropertyChangeListener(this.objectChangeListener);
       }
       home.removeLevelsListener(this.levelsListener);
+      home.removeSelectionListener(this.selectionListener);
     }
   }
   
+  /**
+   * Preferences property listener bound to top camera state with a weak reference to avoid
+   * strong link between user preferences and top camera state.  
+   */
+  private static class UserPreferencesChangeListener implements PropertyChangeListener {
+    private WeakReference<TopCameraState>  topCameraState;
+
+    public UserPreferencesChangeListener(TopCameraState topCameraState) {
+      this.topCameraState = new WeakReference<TopCameraState>(topCameraState);
+    }
+    
+    public void propertyChange(PropertyChangeEvent ev) {
+      // If top camera state was garbage collected, remove this listener from preferences
+      TopCameraState topCameraState = this.topCameraState.get();
+      UserPreferences preferences = (UserPreferences)ev.getSource();
+      if (topCameraState == null) {
+        preferences.removePropertyChangeListener(UserPreferences.Property.valueOf(ev.getPropertyName()), this);
+      } else {
+        topCameraState.setAerialViewCenteredOnSelectionEnabled(preferences.isAerialViewCenteredOnSelectionEnabled());
+      }
+    }
+  }
+
   /**
    * Observer camera controller state. 
    */
